/**
 * @file softSPI.c
 * @brief 波特律动软件SPI驱动
 * @anchor 波特律动(keysking 博哥在学习)
 * @version 1.0
 * @date 2024-07-22
 * @license MIT License
 *
 * @attention
 * 本驱动库针对波特律动·keysking的STM32教程学习套件进行开发
 * 在其他平台或驱动芯片上使用可能需要进行移植
 *
 * @note
 * 使用流程:
 * 1. 填充SoftSPI_TypeDef结构体
 * 2. 调用softSPI_Init初始化SPI（根据第一步结构体的配置进行初始化）
 * 3. 调用SoftSPI_SetCS或SoftSPI_ClrCS设置CS为使能状态（可选）
 * 4. 调用SoftSPI_WriteReadBuff读写数据
 * 5. 调用SoftSPI_SetCS或SoftSPI_ClrCS设置CS为禁能状态（可选）
 *
 */

#include "softSPI.h"

SoftSPI_TypeDef SoftSPI1;

/*
 * @brief  初始化SPI
 * @param  SoftSPIx: 指向SPI结构体的指针（支持多个SPI实例）
 * @retval None
 * 
 * @note   调用该函数前，必须先填充SoftSPI_TypeDef结构体
 * 
 */
void softSPI_Init(SoftSPI_TypeDef *SoftSPIx)
{
    DWT_Delay_Init();
    CS_Set;
    MOSI_Set;
    SCLK_Clr;
}

/*
 * @brief  SPI 读+写数据
 * @param  SoftSPIx: 指向SPI结构体的指针（支持多个SPI实例）
 * @param  pWrite: 写数据缓冲区
 * @param  pRead: 读数据缓冲区
 * @param  len: 数据长度
 * @retval None
 */
void SoftSPI_WriteReadBuff(SoftSPI_TypeDef *SoftSPIx, uint8_t *pWrite, uint8_t *pRead, uint32_t len)
{
    uint8_t data;
    uint8_t byte;
    uint8_t i, j;

    for (j = 0; j < len; j++)
    {
        data = 0;
        byte = pWrite[j];

        if (SoftSPIx->CPHA == 0)
        {
            for (i = 0; i < 8; i++)
            {
                if (byte & 0x80)
                    MOSI_Set;
                else
                    MOSI_Clr;

                Delay;

                byte <<= 1;
                SCLK_Set;

                Delay;

                data <<= 1;
                if (MISO_Read == GPIO_PIN_SET)
                    data |= 0x01;

                Delay;

                SCLK_Clr;

                Delay;
            }
        }
        else
        {
            for (i = 0; i < 8; i++)
            {
                SCLK_Set;

                Delay;

                if (byte & 0x80)
                    MOSI_Set;
                else
                    MOSI_Clr;

                Delay;

                byte <<= 1;
                SCLK_Clr;

                Delay;

                data <<= 1;
                if (MISO_Read == GPIO_PIN_SET)
                    data |= 0x01;

                Delay;
            }
        }

        pRead[j] = data;
    }
}

// 设置CS为高电平
void SoftSPI_CS_High(SoftSPI_TypeDef *SoftSPIx)
{
    CS_Set;
}

// 设置CS为低电平
void SoftSPI_CS_Low(SoftSPI_TypeDef *SoftSPIx)
{
    CS_Clr;
}